@@ -99,6 +99,63 @@ abstract class Stream<T> {
    */
   const factory Stream.empty() = _EmptyStream<T>;
 
+  /**
+   * Creates a stream which emits a single data event before completing.
+   *
+   * This stream emits a single data event of [value]
+   * and then completes with a done event.
+   *
+   * Example:
+   * ```dart
+   * Future<void> printThings(Stream<String> data) async {
+   *   await for (var x in data) {
+   *     print(x);
+   *   }
+   * }
+   * printThings(Stream<String>.value("ok")); // prints "ok".
+   * ```
+   *
+   * The returned stream is effectively equivalent to one created by
+   * `(() async* { yield value; } ())` or `Future<T>.value(value).asStream()`.
+   */
+  @Since("2.5")
+  factory Stream.value(T value) =>
+      (_AsyncStreamController<T>(null, null, null, null)
+            .._add(value)
+            .._closeUnchecked())
+          .stream;
+
+  /**
+   * Creates a stream which emits a single error event before completing.
+   *
+   * This stream emits a single error event of [error] and [stackTrace]
+   * and then completes with a done event.
+   *
+   * Example:
+   * ```dart
+   * Future<void> tryThings(Stream<int> data) async {
+   *   try {
+   *     await for (var x in data) {
+   *       print("Data: $x");
+   *     }
+   *   } catch (e) {
+   *     print(e);
+   *   }
+   * }
+   * tryThings(Stream<int>.error("Error")); // prints "Error".
+   * ```
+   * The returned stream is effectively equivalent to one created by
+   * `Future<T>.error(error, stackTrace).asStream()`, by or
+   * `(() async* { throw error; } ())`, except that you can control the
+   * stack trace as well.
+   */
+  @Since("2.5")
+  factory Stream.error(Object error, [StackTrace stackTrace]) =>
+      (_AsyncStreamController<T>(null, null, null, null)
+            .._addError(error, stackTrace)
+            .._closeUnchecked())
+          .stream;
+
   /**
    * Creates a new single-subscription stream from the future.
    *
@@ -109,7 +166,8 @@ abstract class Stream<T> {
     // Use the controller's buffering to fill in the value even before
     // the stream has a listener. For a single value, it's not worth it
     // to wait for a listener before doing the `then` on the future.
-    _StreamController<T> controller = new StreamController<T>(sync: true);
+    _StreamController<T> controller =
+        new _SyncStreamController<T>(null, null, null, null);
     future.then((value) {
       controller._add(value);
       controller._closeUnchecked();
@@ -136,7 +194,8 @@ abstract class Stream<T> {
    * If [futures] is empty, the stream closes as soon as possible.
    */
   factory Stream.fromFutures(Iterable<Future<T>> futures) {
-    _StreamController<T> controller = new StreamController<T>(sync: true);
+    _StreamController<T> controller =
+        new _SyncStreamController<T>(null, null, null, null);
     int count = 0;
     // Declare these as variables holding closures instead of as
     // function declarations.
@@ -410,6 +469,14 @@ abstract class Stream<T> {
    * The [convert] function is called once per data event per listener.
    * If a broadcast stream is listened to more than once, each subscription
    * will individually call [convert] on each data event.
+   *
+   * Unlike [transform], this method does not treat the stream as
+   * chunks of a single value. Instead each event is converted independently
+   * of the previous and following events, which may not always be correct.
+   * For example, UTF-8 encoding, or decoding, will give wrong results
+   * if a surrogate pair, or a multibyte UTF-8 encoding, is split into
+   * separate events, and those events are attempted encoded or decoded
+   * independently.
    */
   Stream<S> map<S>(S convert(T event)) {
     return new _MapStream<T, S>(this, convert);
@@ -633,6 +700,16 @@ abstract class Stream<T> {
    * Whether the returned stream is a broadcast stream or not,
    * and which elements it will contain,
    * is entirely up to the transformation.
+   *
+   * This method should always be used for transformations which treat
+   * the entire stream as representing a single value
+   * which has perhaps been split into several parts for transport,
+   * like a file being read from disk or being fetched over a network.
+   * The transformation will then produce a new stream which
+   * transforms the stream's value incrementally (perhaps using
+   * [Converter.startChunkedConversion]). The resulting stream
+   * may again be chunks of the result, but does not have to
+   * correspond to specific events from the source string.
    */
   Stream<S> transform<S>(StreamTransformer<T, S> streamTransformer) {
     return streamTransformer.bind(this);
@@ -1484,8 +1561,11 @@ abstract class Stream<T> {
 
     void onData(T event) {
       timer.cancel();
-      controller.add(event);
       timer = zone.createTimer(timeLimit, timeout);
+      // It might close the stream and cancel timer, so create recuring Timer
+      // before calling into add();
+      // issue: https://github.com/dart-lang/sdk/issues/37565
+      controller.add(event);
     }
 
     void onError(error, StackTrace stackTrace) {
@@ -1892,6 +1972,14 @@ abstract class StreamTransformer<S, T> {
    * receives the input stream (the one passed to [bind]) and a
    * boolean flag `cancelOnError` to create a [StreamSubscription].
    *
+   * If the transformed stream is a broadcast stream, so is the stream
+   * returned by the [StreamTransformer.bind] method by this transformer.
+   *
+   * If the transformed stream is listened to multiple times, the [onListen]
+   * callback is called again for each new [Stream.listen] call.
+   * This happens whether the stream is a broadcast stream or not,
+   * but the call will usually fail for non-broadcast streams.
+   *
    * The [onListen] callback does *not* receive the handlers that were passed
    * to [Stream.listen]. These are automatically set after the call to the
    * [onListen] callback (using [StreamSubscription.onData],
